package org.eclipse.palamedes.kif.core.ast;

import org.eclipse.palamedes.kif.core.parser.Parser;

/**
 * Umbrella owner and abstract syntax tree node factory. An AST instance serves
 * as the common owner of any number of AST nodes, and as the factory for
 * creating new AST nodes owned by that instance.
 *
 * @author Ingo Keller
 *
 */
public class AST {

    Parser parser;

    public AST() {
    }

    public KIFComment newComment(String comment) {
        KIFComment newNode = new KIFComment(this);
        newNode.setComment(comment);
        return newNode;
    }

    public KIFConjunction newConjunction(KIFSent sentence, KIFSeq<KIFSent> sequence) {
        KIFConjunction newNode = new KIFConjunction(this);
        newNode.getOperands().add(sentence);
        newNode.getOperands().addAll(sequence);
        return newNode;
    }

    public KIFConjunction newConjunction(KIFSeq<KIFSent> sequence) {
        if (sequence.size() < 2)
            throw new IllegalArgumentException();

        KIFConjunction newNode = new KIFConjunction(this);
        newNode.getOperands().addAll(sequence);
        return newNode;
    }

    public KIFDisjunction newDisjunction(KIFSent sentence, KIFSeq<KIFSent> sequence) {
        KIFDisjunction newNode = new KIFDisjunction(this);
        newNode.getOperands().add(sentence);
        newNode.getOperands().addAll(sequence);
        return newNode;
    }

    public KIFDisjunction newDisjunction(KIFSeq<KIFSent> sequence) {
        if (sequence.size() < 2)
            throw new IllegalArgumentException();

        KIFDisjunction newNode = new KIFDisjunction(this);
        newNode.getOperands().addAll(sequence);
        return newNode;
    }

    public KIFForm newForm() {
        return new KIFForm(this);
    }

    public KIFForm newForm(KIFSeq<KIFSent> sentences) {
        KIFForm newNode = new KIFForm(this);
        newNode.getChildren().addAll(sentences);
        for (KIFSent sentence : sentences)
            if (sentence instanceof KIFComment)
                newNode.getComments().add((KIFComment) sentence);
            else
                newNode.getSentences().add(sentence);
        return newNode;
    }

    public KIFFunTerm newFunTerm(String funConst) {
        KIFFunTerm newNode = new KIFFunTerm(this);
        newNode.setConstName(funConst);
        return newNode;
    }

    public KIFFunTerm newFunTerm(String funConst, KIFSeq<KIFTerm> termSeq) {
        KIFFunTerm newNode = new KIFFunTerm(this);
        newNode.setConstName(funConst);
        newNode.getTerms().addAll(termSeq);
        return newNode;
    }

    public KIFImplication newImplication(KIFSent sentence, KIFSeq<KIFSent> sequence) {
        KIFImplication newNode = new KIFImplication(this);
        newNode.setConsequence(sentence);
        newNode.getPremise().addAll(sequence);
        return newNode;
    }

    public KIFImplication newImplication(KIFSent sentence) {
        KIFImplication newNode = new KIFImplication(this);
        newNode.setConsequence(sentence);
        return newNode;
    }

    public KIFIndVar newIndVar(String varName) {
        KIFIndVar newNode = new KIFIndVar(this);
        newNode.setVarName(varName);
        return newNode;
    }

    public KIFLogConst newLogConst(String constName) {
        KIFLogConst newNode = new KIFLogConst(this);
        newNode.setConstName(constName);
        return newNode;
    }

    public KIFNegation newNegation(KIFSent sentence) {
        KIFNegation newNode = new KIFNegation(this);
        newNode.getOperands().add(sentence);
        return newNode;
    }

    public KIFObjConst newObjConst(String constName) {
        KIFObjConst newNode = new KIFObjConst(this);
        newNode.setConstName(constName);
        return newNode;
    }

    public KIFOperator newOperator(String operator) {
        KIFOperator newNode = new KIFOperator(this);
        newNode.setOperator(operator);
        return newNode;
    }

    public KIFRelSent newRelSent(String constName, KIFSeq<KIFTerm> terms) {
        KIFRelSent newNode = new KIFRelSent(this);
        newNode.setConstName(constName);
        if (terms != null)
            newNode.getTerms().addAll(terms);
        return newNode;
    }

    public KIFSeq<KIFSent> newSeq(KIFSent sentence) {
        KIFSeq<KIFSent> newNode = new KIFSeq<KIFSent>();
        newNode.add(sentence);
        return newNode;
    }

    public KIFSeq<KIFSent> newSeq(KIFSeq<KIFSent> sequence, KIFSent sentence) {
        KIFSeq<KIFSent> newNode = new KIFSeq<KIFSent>();
        newNode.addAll(sequence);
        newNode.add(sentence);
        return newNode;
    }

    public KIFSeq<KIFTerm> newSeq(KIFTerm term) {
        KIFSeq<KIFTerm> newNode = new KIFSeq<KIFTerm>();
        newNode.add(term);
        return newNode;
    }

    public KIFSeq<KIFTerm> newSeq(KIFSeq<KIFTerm> sequence, KIFTerm term) {
        KIFSeq<KIFTerm> newNode = new KIFSeq<KIFTerm>();
        newNode.addAll(sequence);
        newNode.add(term);
        return newNode;
    }

    public void modifying() {
    }

    public Parser getParser() {
        if (parser == null)
            parser = new Parser(this);
        return parser;
    }

    public ASTNode parseString(String text) {
        return (ASTNode) getParser().parseGDL(text);
    }

    public ASTNode parseFile(String filename) {
        return (ASTNode) getParser().parseFile(filename);
    }

}
