package org.eclipse.palamedes.kif.core.ast.visitor;

import org.eclipse.palamedes.kif.core.ast.ASTNode;
import org.eclipse.palamedes.kif.core.ast.ASTVisitor;
import org.eclipse.palamedes.kif.core.ast.KIFConjunction;
import org.eclipse.palamedes.kif.core.ast.KIFDisjunction;
import org.eclipse.palamedes.kif.core.ast.KIFForm;
import org.eclipse.palamedes.kif.core.ast.KIFFunTerm;
import org.eclipse.palamedes.kif.core.ast.KIFImplication;
import org.eclipse.palamedes.kif.core.ast.KIFIndVar;
import org.eclipse.palamedes.kif.core.ast.KIFLogConst;
import org.eclipse.palamedes.kif.core.ast.KIFNegation;
import org.eclipse.palamedes.kif.core.ast.KIFObjConst;
import org.eclipse.palamedes.kif.core.ast.KIFOperator;
import org.eclipse.palamedes.kif.core.ast.KIFRelSent;
import org.eclipse.palamedes.kif.core.ast.KIFSent;
import org.eclipse.palamedes.kif.core.ast.KIFSeq;
import org.eclipse.palamedes.kif.core.ast.KIFTerm;



public class NotationWriter {

	private static NotationWriter instance = new NotationWriter();
	
    public static final int INFIX  = 1;
    public static final int PREFIX = 2;

    private Visitor writer = new Visitor();

    private static class Visitor extends ASTVisitor {

        private StringBuilder sb;
        private int           _type = INFIX;


        @Override
        public void visit(KIFConjunction node) {
            switch (_type) {
            case INFIX:
                sb.append("(").append(node.getName()).append(" ");
                break;
            case PREFIX:
                sb.append(node.getName()).append("(");
                break;
            }
            for (KIFSent child : node.getOperands()) {
                child.traverse(this);
                sb.append(" ");
            }

            if (node.getOperandCount() > 0)
                sb.deleteCharAt(sb.length() - 1);

            sb.append(")");
        }

        @Override
        public void visit(KIFDisjunction node) {
            switch (_type) {
            case INFIX:
                sb.append("(").append(node.getName()).append(" ");
                break;
            case PREFIX:
                sb.append(node.getName()).append("(");
                break;
            }
            for (KIFSent child : node.getOperands()) {
                child.traverse(this);
                sb.append(" ");
            }

            if (node.getOperandCount() > 0)
                sb.deleteCharAt(sb.length() - 1);

            sb.append(")");
        }

        @Override
        public void visit(KIFForm node) {
            for (KIFSent child : node.getSentences()){
            	child.traverse(this);
            	sb.append(" ");
            }
                
        }

        @Override
        public void visit(KIFFunTerm node) {
            switch (_type) {
            case INFIX:
                sb.append("(").append(node.getName()).append(" ");
                break;
            case PREFIX:
                sb.append(node.getName()).append("(");
                break;
            }
            for (KIFTerm child : node.getTerms()) {
                child.traverse(this);
                sb.append(" ");
            }

            if (node.getTerms().size() > 0)
                sb.deleteCharAt(sb.length() - 1);

            sb.append(")");
        }

        @Override
        public void visit(KIFImplication node) {
            switch (_type) {
            case INFIX:
                sb.append("(").append(node.getName()).append(" ");
                break;
            case PREFIX:
                sb.append(node.getName()).append("(");
                break;
            }

            node.getConsequence().traverse(this);
            sb.append("\n");

            for (KIFSent child : node.getPremise()) {
                child.traverse(this);
                sb.append("\n");
            }

            sb.deleteCharAt(sb.length() - 1);
            sb.append(")");
        }

        @Override
        public void visit(KIFIndVar node) {
            sb.append(node.getName());
        }

        @Override
        public void visit(KIFLogConst node) {
            sb.append(node.getName());
        }

        @Override
        public void visit(KIFNegation node) {
            switch (_type) {
            case INFIX:
                sb.append("(").append(node.getName()).append(" ");
                break;
            case PREFIX:
                sb.append(node.getName()).append("(");
                break;
            }
            node.getOperand().traverse(this);
            sb.append(")");
        }

        @Override
        public void visit(KIFObjConst node) {
            sb.append(node.getName());
        }

        @Override
        public void visit(KIFOperator node) {
            sb.append(node.getOperator());
        }

        @Override
        public void visit(KIFRelSent node) {

            // GDL workaround
            if (node.getOperandCount() == 0) {
                sb.append(node.getName());
                return;
            }

            switch (_type) {
            case INFIX:
                sb.append("(").append(node.getName()).append(" ");
                break;
            case PREFIX:
                sb.append(node.getName()).append("(");
                break;
            }
            for (KIFTerm child : node.getOperands()) {
                child.traverse(this);
                sb.append(" ");
            }
            if (node.getOperandCount() > 0)
                sb.deleteCharAt(sb.length() - 1);

            sb.append(")");
        }

        public void reset() {
            sb = new StringBuilder();
        }

        public void setType(int type) {
            _type = type;
        }

		public int getType() {
			return _type;
		}

		@Override
        public String toString() {
            return sb.toString();
        }

    }


    public static NotationWriter getInstance() {
    	return instance;
    }
    
    public static String toDefaultString(ASTNode node) {
    	return instance.toString(node);
    }
    
    public static String toDefaultString(KIFSeq<? extends ASTNode> sequence) {
    	return instance.toString(sequence);
    }
    
    public static String toDefaultString(KIFSeq<? extends ASTNode> sequence, String filler) {
    	return instance.toString(sequence,filler);
    }

    public static void setType(int type) {
        instance.writer.setType(type);
    }

    public NotationWriter(){
    }
    
    public NotationWriter(int type){
    	writer.setType(type);
    }
    
    public String toString(KIFSeq<? extends ASTNode> sequence) {
        StringBuilder sb = new StringBuilder();
        for (ASTNode node : sequence)
            sb.append( toDefaultString(node) ).append(" ");

        return sb.deleteCharAt(sb.length() - 1).toString();
    }

    public String toString(KIFSeq<? extends ASTNode> sequence, String filler) {
        StringBuilder sb = new StringBuilder();
        for (ASTNode node : sequence)
            sb.append( toDefaultString(node) ).append(filler);

        // remove last filler
        final int fillLength = filler.length();
        final int sbLength   = sb.length() - 1;
        sb.delete(sbLength - fillLength, sbLength);

        return sb.toString();
    }

    public String toString(ASTNode node) {
        writer.reset();

        switch (node.getNodeType()) {
        case ASTNode.CONJUNCTION:
            writer.visit( (KIFConjunction) node );
            break;
        case ASTNode.DISJUNCTION:
            writer.visit( (KIFDisjunction) node );
            break;
        case ASTNode.FORM:
            writer.visit( (KIFForm) node );
            break;
        case ASTNode.FUNTERM:
            writer.visit( (KIFFunTerm) node );
            break;
        case ASTNode.IMPLICATION:
            writer.visit( (KIFImplication) node );
            break;
        case ASTNode.INDVAR:
            writer.visit( (KIFIndVar) node );
            break;
        case ASTNode.LOGCONST:
            writer.visit( (KIFLogConst) node );
            break;
        case ASTNode.NEGATION:
            writer.visit( (KIFNegation) node );
            break;
        case ASTNode.OBJCONST:
            writer.visit( (KIFObjConst) node );
            break;
        case ASTNode.OPERATOR:
            writer.visit( (KIFOperator) node );
            break;
        case ASTNode.RELSENT:
            writer.visit( (KIFRelSent) node );
            break;
        }
        return writer.toString();
    }

}
